#include "xplodeconfig.h"

#include <qdir.h>
#include <qprogressdialog.h>
#include <qmsgbox.h>

// Shared variables
QString       *plasmXplode;
QString       *plasmPath;
QString       *plasmVrmlViewer;
QString       *plasmSvgViewer;
QString       *plasmDock;
QFont         *plasmEditorFont;
QFont         *plasmListenerFont;
PlasmKeywords *plasmKeywords;

extern QProgressDialog *initDialog;

// Constructor
XplodeConfig::XplodeConfig(QString path)
{
        // Construct the configuration file path for the current operating system
#ifdef linux
        // Get the user home directory
        configFile = new QString(getenv("HOME"));
        // Build the configuration file path
        configFile->append("/.xploderc.xml");
#endif

#ifdef WIN32
        // Get the user home directory
        configFile = new QString(getenv("USERPROFILE"));
        // If "USERPROFILE" is not available, then we're running Windows 9x/Me
        if (configFile->isEmpty()) *configFile = "C:\\WINDOWS";
        configFile->append("\\xploderc.xml");
#endif

#ifdef __APPLE__
        // Get the user home directory
        configFile = new QString(getenv("HOME"));
        // Build the configuration file path
        configFile->append("/.xploderc.xml");
#endif
        plasmXplode = new QString(path);
        initDialog->setProgress(10);
}

// Destructor
XplodeConfig::~XplodeConfig()
{
        delete configFile;
}

// Read the configuration file and set the shared variables
bool XplodeConfig::readConfig()
{
        initDialog->setProgress(1);
        initDialog->setLabelText("Reading configuration...");
        // Open the configuration file
        QFile config(*configFile);
        // If an error occurs, exit
        if (!config.open(IO_ReadOnly)) return false;
        // Build the XML document
        configDoc = new QDomDocument();
        // Parse the XML configuration. On error, exit
        if (!configDoc->setContent(&config)) return false;
        // Close the file
        config.close();

        // DOM variables
        QDomNodeList l;
        QDomElement elem;
        QDomNamedNodeMap attribs;

        initDialog->setLabelText("Parsing configuration...");
        // Load values from the document
        l = configDoc->elementsByTagName("plasm-path");
        if (l.count() > 0)
        {
                plasmPath = new QString(l.item(0).attributes().namedItem("path").toAttr().value());
        }

        l = configDoc->elementsByTagName("editor-font");
        if (l.count() > 0)
        {
                attribs = l.item(0).attributes();
                plasmEditorFont = new QFont(attribs.namedItem("family").toAttr().value(),
                                            attribs.namedItem("size").toAttr().value().toInt(),
                                            attribs.namedItem("weight").toAttr().value().toInt());
        }

        l = configDoc->elementsByTagName("listener-font");
        if (l.count() > 0)
        {
                attribs = l.item(0).attributes();
                plasmListenerFont = new QFont(attribs.namedItem("family").toAttr().value(),
                                              attribs.namedItem("size").toAttr().value().toInt(),
                                              attribs.namedItem("weight").toAttr().value().toInt());
        }

        l = configDoc->elementsByTagName("vrml-viewer");
        if (l.count() > 0)
        {
                plasmVrmlViewer = new QString(l.item(0).attributes().namedItem("path").toAttr().value());
        }

        l = configDoc->elementsByTagName("svg-viewer");
        if (l.count() > 0)
        {
                plasmSvgViewer = new QString(l.item(0).attributes().namedItem("path").toAttr().value());
        }

        // Now all shared variables should be set to non-null. If not, exit
        if (!plasmEditorFont ||
            !plasmListenerFont ||
            !plasmPath ||
            !plasmSvgViewer ||
            !plasmVrmlViewer) return false;

        initDialog->setProgress(20);

        // Construc the keywords
        QDir d(*plasmXplode);
        QString absPath = d.absPath();
        // Get the directory
#ifdef WIN32
        QString thePath = absPath.left(absPath.length() - 10);
#else
        QString thePath = absPath.left(absPath.length() - 6);
#endif
        initDialog->setLabelText("Reading keywords...");
        // Initialize the keywords
        plasmKeywords = new PlasmKeywords();
        QFile kwords(thePath + "keywords.xml");
        QString currentLib = "";
        KeyWord *currentKeyword;
        QDomDocument *funcList = new QDomDocument();
        // Open the XML file
        if (kwords.open(IO_ReadOnly) && funcList->setContent(&kwords))
        {
                // Document root
                QDomNode root = funcList->elementsByTagName("plasm-keywords").item(0);
                // Not null root (well formed document)
                if (!root.isNull())
                {
                        // Get the first child
                        QDomNode library = root.firstChild();
                        // for each library
                        while (!library.isNull())
                        {
                                if (library.isElement())
                                {
                                        QDomElement libraryEl = library.toElement();
                                        // create a library node
                                        currentLib = libraryEl.attribute("name");
                                        QDomNode node = library.firstChild();
                                        // loop over all childs
                                        while (!node.isNull())
                                        {
                                                if (node.isElement())
                                                {
                                                        QDomElement functionEl = node.toElement();
                                                        currentKeyword = new KeyWord(functionEl.attribute("name").lower(),
                                                                                     currentLib,
                                                                                     functionEl.attribute("input"),
                                                                                     functionEl.attribute("output"),
                                                                                     functionEl.attribute("description"));
                                                        plasmKeywords->db[currentKeyword->getName().lower()] = currentKeyword;
                                                }
                                                node = node.nextSibling();
                                        }
                                }
                                library = library.nextSibling();
                        }
                }
        }
        else
        {
                QMessageBox::warning(0, "Xplode - Config", "Can't find the keyword database file.\nAre you sure you've done a plain installation?", QMessageBox::Yes, QMessageBox::NoButton);
                return false;
        }
        return true;
}

// Save the configuration file
bool XplodeConfig::saveConfig()
{
        // Open the configuration file and connect it to a text stream
        QFile config(*configFile);
        QTextStream stream(&config);
        // If an error occurs, exit
        if (!config.open(IO_WriteOnly)) return false;

        configDoc = new QDomDocument();
        QDomElement root = configDoc->createElement("xplode-config");
        configDoc->appendChild(root);

        QDomElement element = configDoc->createElement("plasm-path");
        element.setAttribute("path", *plasmPath);
        root.appendChild(element);

        element = configDoc->createElement("editor-font");
        element.setAttribute("family", plasmEditorFont->family());
        element.setAttribute("size", plasmEditorFont->pointSize());
        element.setAttribute("weight", plasmEditorFont->weight());
        root.appendChild(element);

        element = configDoc->createElement("listener-font");
        element.setAttribute("family", plasmListenerFont->family());
        element.setAttribute("size", plasmListenerFont->pointSize());
        element.setAttribute("weight", plasmListenerFont->weight());
        root.appendChild(element);

        element = configDoc->createElement("vrml-viewer");
        element.setAttribute("path", *plasmVrmlViewer);
        root.appendChild(element);

        element = configDoc->createElement("svg-viewer");
        element.setAttribute("path", *plasmSvgViewer);
        root.appendChild(element);

        // Write the xml file
        QString xml = configDoc->toString();
        stream << xml;
        config.close();

        return true;
}

// Creates a standard configuration file
bool XplodeConfig::rebuildConfig()
{
        QFileInfo* f = new QFileInfo(*plasmXplode);
        // Get the Xplode executable directory
        QString dir = f->absFilePath().left(f->absFilePath().length() - f->fileName().length());
        QString suffix = QString(""), editorFont, listenerFont;

#ifdef linux
        // Set the default font names
        editorFont = "Monospace";
        listenerFont = "Courier";
#endif

#ifdef __APPLE__
        // Set the default font names
        editorFont = "Mono";
        listenerFont = "Courier";
#endif

#ifdef WIN32
        // Set the default font names
        editorFont = "Mono";
        listenerFont = "Courier New";
        // File suffixes are needed only for windows
        suffix = ".exe";
#endif

        // Set the default variables
        plasmPath = new QString(dir + "plasm" + suffix);
        plasmVrmlViewer = new QString(dir + "vrmlview" + suffix);
        plasmSvgViewer = new QString(dir + "vrmlview" + suffix);
        plasmEditorFont = new QFont(editorFont, 10);
        plasmListenerFont = new QFont(listenerFont, 10);

        // Now save it
        return saveConfig();
}
