/**********************************************************************
** This file is part of QMWEdit Project. This file has been slightly
** modified for Xplode. The original project can be found at:
**
** http://sourceforge.net/
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
**********************************************************************/

#include "xplodeeditor.h"
#include "xplodeicons.h"

// This file has been slightly modified for Xplode

// Constructor
XplodeEditor::XplodeEditor(QWidget *parent, const char *name) : Editor(parent, name)
{
        // Variable initialization
        para = index = 0;
        scan_flag = true;
        complete_flag = false;
        nr_of_letters = 0;
        scan_word = "";
        word = " ";

        // Initialize the text parser/formatter
        document()->setPreProcessor(new XplodeSyntaxHighlighter);

        // Initialize the configuration
        configChanged();
        // No word wrap
        setWordWrap(NoWrap);
        // Set the cursor position
        setCursorPosition(0,0);
        s_NewCursorPos(0,0);
        // Let's have QTextEdit decide aboud scroll bars
        setVScrollBarMode(QScrollView::Auto);
        setHScrollBarMode(QScrollView::Auto);
        // No overwriting
        setOverwriteMode(false);
        emit s_overWriteMode(false);
        connect(this, SIGNAL(doubleClicked(int, int)), this, SLOT(doubleClickEvent(int, int)));
}
// Destructor
XplodeEditor::~XplodeEditor()
{
}

// Set the cursor position, emitting a signal
void XplodeEditor::setCursorPos(int p_para, int p_index)
{
        para = p_para;
        index = p_index;
        setCursorPosition(para, index);
        emit s_NewCursorPos(para, index);
}

// Get the cursor position
void XplodeEditor::getCursorPos(int *p_para, int *p_index)
{
        *p_para = para;
        *p_index = index;
}

// Handle key-press events
void XplodeEditor::keyPressEvent(QKeyEvent *e)
{
        int key=e->key();
        switch(key)
        {
                // Switch insert/overwrite mode
                case Key_Insert:
                        if (overWriteMode)
                        {
                                overWriteMode = false;
                                setOverwriteMode(false);
                                emit s_overWriteMode(false);
                        }
                        else
                        {
                                overWriteMode = true;
                                setOverwriteMode(true);
                                emit s_overWriteMode(true);
                        }
                        break;
        }
        QTextEdit::keyPressEvent(e);
        getCursorPosition(&para ,&index);
        emit s_NewCursorPos(para, index);
}

// Mouse events. This has been added for future implementations
void XplodeEditor::mousePressEvent(QMouseEvent *m)
{
        if (m->button() == LeftButton)
        {
                // Code completion: future implementations?
                complete_flag = false;
                nr_of_letters = 0;
                scan_word = "";
                word = " ";
                // Now, just call the right function
                QTextEdit::mousePressEvent(m);
                getCursorPosition(&para, &index);
                emit s_NewCursorPos(para, index);
        }
}

void XplodeEditor::doubleClickEvent(int p, int i)
{
        int startIdx, endIdx;
        QString paragraph = text(p);
        // Get the type of the selected char
        int what = paragraph.at(i).category();
        // Search for the first token to the right
        startIdx = endIdx = i;
        while ((startIdx >= 0) && (paragraph.at(startIdx).category() == what))
                startIdx--;
        // Search for the last token to the left
        while ((endIdx <= paragraph.length()) && (paragraph.at(endIdx).category() == what))
                endIdx++;
        setSelection(p, startIdx, p, endIdx);
}

// Creates a new configuration
void XplodeEditor::configChanged()
{
        QMap<QString, ConfigStyle> styles = Config::defaultStyles();
        config()->styles = styles;
        ((XplodeSyntaxHighlighter*)document()->preProcessor())->updateStyles(config()->styles);
        parenMatcher->setEnabled(true);
        setFont(((XplodeSyntaxHighlighter*)document()->preProcessor())->format(0)->font());
        Editor::configChanged();
}

void XplodeEditor::contentsMousePressEvent(QMouseEvent *event)
{
        // Store the drag position
        if (event->button() == LeftButton)
                dragPos = event->pos();
        Editor::contentsMousePressEvent(event);
}

void XplodeEditor::contentsMouseMoveEvent(QMouseEvent *event)
{
        // Are we still pressing the left button?
        if (event->state() & LeftButton)
                startDrag();
        Editor::contentsMouseMoveEvent(event);
}

void XplodeEditor::startDrag()
{
        // Get the selected text
        QString text = selectedText();
        if (!text.isEmpty())
        {
                // Create the drag item
                QTextDrag *dragText = new QTextDrag(text, this);
                // Cool pixmap
                dragText->setPixmap(QPixmap(qembed_findImage("plasmevalbuffer")));
                dragText->dragCopy();
        }
}

void XplodeEditor::contentsDragEnterEvent (QDragEnterEvent *event)
{
        if (event->source() == this)
                event->ignore();
        else
                event->accept();
}

void XplodeEditor::contentsDragMoveEvent (QDragMoveEvent *event)
{
        if (event->source() == this)
                event->ignore();
        else
                event->accept();
}

void XplodeEditor::contentsDropEvent (QDropEvent *event)
{

        if (event->source() == this)
                event->ignore();
        else
        {
                QString text;
                event->accept();
                if (QTextDrag::decode(event, text))
                {
                        insert(text);
                }
        }
}
