/**********************************************************************
** This file is part of QMWEdit Project. This file has been slightly
** modified for Xplode. The original project can be found at:
**
** http://sourceforge.net/
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
**********************************************************************/

#ifndef XPLODEEDITOR_H
#define XPLODEEDITOR_H

// This file has been slightly modified for Xplode

// Qt includes
#include <qcolor.h>
#include <qkeycode.h>
#include <qobjectlist.h>
#include <qpainter.h>
#include <qpopupmenu.h>
#include <qptrlist.h>
#include <qsettings.h>
#include <qevent.h>
#include <qdragobject.h>
#include <qstring.h>
#include <qtextedit.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qdir.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qsizegrip.h>
#include <qspinbox.h>

// Xplode includes
#include "editor.h"
#include "xplodesyntaxhighlighter.h"
#include "parenmatcher.h"

// The real editor window, with syntax highlight
class XplodeEditor : public Editor
{
        Q_OBJECT

private:
        int para;
        int index;

        int languageMode;
        bool overWriteMode;
        bool hexEditMode;
        bool wordWrapMode;
        bool readOnlyMode;
        bool lineNumMode;
        bool spaceTabMode;

public:
        XplodeEditor(QWidget *parent, const char *name);
        virtual ~XplodeEditor();
        void contentsChanged() { emit textChanged(); }
        bool markedText() { return hasSelectedText(); }
        void configChanged();

        void setCursorPos(int p_para, int p_index);
        void getCursorPos(int *p_para, int *p_index);

        void setOverWriteMode(bool OVERWRITE){ overWriteMode = OVERWRITE; }
        bool getOverWriteMode(){ return overWriteMode; }

private:
    // For the scan_word function
        bool scan_flag;
        bool complete_flag;
        int nr_of_letters;
        QString scan_word;
        QString word;
        QPoint dragPos;

protected:
        virtual void keyPressEvent(QKeyEvent *);
        void startDrag();
        virtual void contentsMousePressEvent(QMouseEvent *event);
        virtual void contentsMouseMoveEvent(QMouseEvent *event);
        virtual void contentsDragEnterEvent (QDragEnterEvent *event);
        virtual void contentsDragMoveEvent (QDragMoveEvent *event);
        virtual void contentsDropEvent (QDropEvent *event);

public slots:
        void mousePressEvent(QMouseEvent *);
        void doubleClickEvent(int p, int i);

signals:
        void s_NewCursorPos(int, int);
        void s_overWriteMode(bool);
};

#endif
